%
%CHORD2PITCHCLASSES Convert chord symbol to array of pitch classes
% 
% [pitchclasses, bassclass, success, errormessage] = chord2pitchclasses(chordsymbol, {verbose})
% 
% Converts a chord symbol to an array of integer pitch class values and a
% bass pitch class value. Pitch class values are referenced w.r.t. C = 0.
%
% In the case of the 'no chord' symbol 'N' function returns an empty array
%
% Success = 1 if notes extracted from chordsymbol correctly, 0 otherwise.
% 
% If optional argument 'verbose' is 1, function prints any errormessage to 
% the screen.
% 
% calls:    chord2notes
%           notes2pitchclasses
% 
% returns:  pitchclasses (array of integers)
%           bassclass (integer)           
%           success (boolean)
%           errormessage (string)
%
%
% Author: Christopher Harte,  August 2005
% 
% Copyright: Centre for Digital Music, Queen Mary University of London 2005 
%
% This file is part of the C4DM Chord Toolkit.  
%
% The C4DM Chord Toolkit is free software; you can redistribute it and/or 
% modify it under the terms of the GNU General Public License as published 
% by the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% The C4DM Chord Toolkit is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with the C4DM Toolkit; if not, write to the Free Software
% Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

%             
function [pitchclasses, bassclass, success, errormessage] = chord2pitchclasses(chordsymbol, verbose)

% set verbose default to 0
if nargin < 2
    verbose = 0;
end

errormessage = '';
mainlist = '';
success = 1;
pitchclasses = [];
bassclass = '';

% get constituent notes
[chordnotes, bassnote, success, errormessage] = chord2notes(chordsymbol);

if success && ~isempty(chordnotes)
% find pitchclasses of notes
[pitchclasses, success, errormessage] = notes2pitchclasses(chordnotes);
end

if success && ~isempty(bassnote)
[bassclass, success, errormessage] = note2pitchclass(char(bassnote));
end

if success == 0
    errormessage = [errormessage sprintf(['Error in chord2pitchclasses: Couldn''t convert chord"' chordsymbol '"\n'])];   
    if verbose ==1
       fprintf(1,errormessage);
    end
       
       
end  
    
    
    