%
%NOTES2PITCHCLASSES convert a note list to an array pitchclasses
% 
% [pitchclasses, success, errormessage] = notes2pitchclasses(notes, {verbose})
% 
% Converts a cell array of note strings to an array of pitch class integer 
% values. Pitch class values are referenced w.r.t. C = 0. 
% 
% Success = 1 if symbols converted correctly, 0 otherwise.
% 
% If optional argument 'verbose' is 1, function prints any errormessage to 
% the screen.
% 
% calls note2pitchclass
% 
% returns:  pitchclasses (array of integers)
%           success (boolean)
%           errormessage (string)
%
%
% Author: Christopher Harte,  August 2005
% 
% Copyright: Centre for Digital Music, Queen Mary University of London 2005 
%
% This file is part of the C4DM Chord Toolkit.  
%
% The C4DM Chord Toolkit is free software; you can redistribute it and/or 
% modify it under the terms of the GNU General Public License as published 
% by the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% The C4DM Chord Toolkit is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with the C4DM Toolkit; if not, write to the Free Software
% Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
%

function [pitchclasses, success, errormessage] = notes2pitchclasses(notes, verbose)

% set verbose default to 0
if nargin < 2
    verbose = 0;
end

errormessage = '';

ilength = length(notes);

index = 1;

success = 1;

pitchclasses = [];

while index <= ilength
    
        [pitchclasses(index), success, errormessage] = note2pitchclass(char(notes(index)));
    
        if success
            index=index+1;
        else
            errormessage = [errormessage sprintf(['Error in notes2pitchclasses: couldn''t convert notes "' notes '"\n'])];
            index = ilength +1;            
        end
    
    
end

if (success == 0) && (verbose == 1)

    fprintf(1,errormessage)
    
end    



