%
%SYNTAXCHECK Checks symbol for correct syntax. 
% 
% [success, errormessage] = syntaxcheck(chordsymbol, {verbose})
% 
% Checks symbol for correct syntax and checks for valid note, shorthand 
% and interval values. 
%
% calls getchordinfo
% 
% Success = 1 if symbol has correct syntax, 0 otherwise.
% 
% If optional argument 'verbose' is 1, function prints any errormessage to 
% the screen.
% 
% Returns:  chordinfo (struct)
%           success (boolean) true or false for correct syntax 
%           errormessage (string)  
%     
% See also: getchordinfo
%
% Author: Christopher Harte,  August 2005
% 
% Copyright: Centre for Digital Music, Queen Mary University of London 2005 
%
% This file is part of the C4DM Chord Toolkit.  
%
% The C4DM Chord Toolkit is free software; you can redistribute it and/or 
% modify it under the terms of the GNU General Public License as published 
% by the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% The C4DM Chord Toolkit is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with the C4DM Toolkit; if not, write to the Free Software
% Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

%		
function [success, errormessage] = syntaxcheck(chordsymbol, verbose)

if nargin < 2
    verbose = 0;
end
    
errormessage = '';
chordinfo = [];

success = 0;

[root,short,degrees,bass, success, errormessage] = getchordinfo(chordsymbol,verbose);

if (success ==0) && (verbose == 1)
    fprintf(1,errormessage);
end