function bnet = dbn_thesis_mbk24bx(param)

node_sizes = [param.dbn.nMetpos param.dbn.nKey param.dbn.nChord 13 13 12];
discrete = 1:4;
observed = 5:6;
%% topology: metric position

nodenames = {'metpos', 'key', 'chord', 'bass' , 'basschroma','treblechroma'};

%% topology: dependencies
% I first create full dependency matrices, and then eliminate the
% inapproapriate rows and columns

intra = zeros(6);
inter = zeros(6);

intra(1,3) = 1; % metpos to chord
intra(2,3) = 1; % key to chord
intra(3,4) = 1; % chord to bass
intra(3,6) = 1; % chord to treble chroma
intra(4,5) = 1; % bass to bass chroma

inter(1,1) = 1; % metpos to metpos
inter(2,2) = 1; % key to key
inter(3,3) = 1; % chord to chord
inter(3,4) = 1; % chord to bass

bnet = mk_dbn(intra, inter, ...
    node_sizes, 'discrete', discrete, 'observed', observed, ...
    'names', nodenames);

%-------------------------------------------------------
% Flat initialisation for hidden nodes in first slice
%-------------------------------------------------------
fprintf(1,'   - First Slice\n');
iNode = 1;
for nodename = nodenames(discrete)
    fprintf(1,'    %s \n', nodename{1});
    bnet.CPD{iNode} = tabular_CPD(bnet, iNode, 'CPT', 'unif');
    iNode = iNode + 1;
end

%-------------------------------------------------------
% Observed nodes
%-------------------------------------------------------
bnet.CPD{bnet.names('basschroma')} = ...
    gaussian_CPD(bnet,bnet.names('basschroma'), ...
    'mean', eye(13), 'cov', repmat(eye(13) * sqrt(param.dbn.basschromavar),[1,1,13]));
chordpcs = param.dbn.chordpcs;
chordpcs(:,end) = chordpcs(:,end) - 0.5;
cova = param.dbn.cova;
cova(:,:,end) = cova(:,:,end);
bnet.CPD{bnet.names('treblechroma')} = ...
    gaussian_CPD(bnet,bnet.names('treblechroma'), ...
    'mean', chordpcs, 'cov', cova);

%-------------------------------------------------------
% second slice -- more elaborate stuff ...
%-------------------------------------------------------

fprintf(1,'   - Second Slice\n');
nodename = 'metpos';
nodenumber = bnet.names(nodename) + bnet.nnodes_per_slice;
fprintf(1,'    (%d) %s (slice 2) \n', nodenumber, nodename);
% columns are:
% (1) old metpos,
% (2) this metpos,
% (3) conditional probability.

helper_table = get_field(tabular_CPD(bnet,nodenumber,'CPT','unif'),'cpt');
helper_table = helper_table * 0;
for oldmetpos = 1:param.dbn.nMetpos
    for thismetpos = 1:param.dbn.nMetpos
        if oldmetpos <= 4
            if mod(oldmetpos + 1,4)  == ...
                    mod(thismetpos,4)
                helper_table(oldmetpos,thismetpos) = param.dbn.beattrans;
            else
                helper_table(oldmetpos,thismetpos) = (1-param.dbn.beattrans)/3;
            end
        else
            helper_table(oldmetpos,thismetpos) = 1/param.dbn.nMetpos;
        end
    end
end

bnet.CPD{nodenumber} = tabular_CPD(bnet,nodenumber,helper_table);

%-------------------------------------------------------

nodename = 'key';
nodenumber = bnet.names(nodename) + bnet.nnodes_per_slice;
fprintf(1,'    (%d) %s (slice 2) \n', nodenumber, nodename);
% columns are
% (1) old key
% (2) this key
% (3) probability.

helper_table = get_field(tabular_CPD(bnet,nodenumber,'CPT','unif'),'cpt');

for iOld = 1:param.dbn.nKey
    for iThis = 1:param.dbn.nKey
        if iOld == iThis
            helper_table(iOld,iThis) = 1-param.dbn.keychange;
        else
            helper_table(iOld,iThis) = param.dbn.keychange/(param.dbn.nKey-1);
        end
    end
end

bnet.CPD{nodenumber} = tabular_CPD(bnet,nodenumber,helper_table);

%-------------------------------------------------------
nodename = 'chord';
nodenumber = bnet.names(nodename) + bnet.nnodes_per_slice;
fprintf(1,'    (%d) %s (slice 2) \n', nodenumber, nodename);
% columns are:
% (1) old chord,
% (2) new metpos,
% (3) new harmonic meter,
% () new key,
% (5) this chord,
% (6) probability.

helper_table = get_field(tabular_CPD(bnet,nodenumber,'CPT','unif'),'cpt');
[cgivenk, param] = chordgivenkey(param);

for iKey = 1:param.dbn.nKey
    for iThis = 1:param.dbn.nChord
        for iBeat = 1:param.dbn.nMetpos
            for iOld = 1:param.dbn.nChord
                if iOld == iThis
                    helper_table(iOld,iBeat,iKey,iThis) = ...
                        param.dbn.chordchange(iBeat) * cgivenk(iKey, iThis);
                else
                    helper_table(iOld,iBeat,iKey,iThis) = ...
                        (1-param.dbn.chordchange(iBeat)) / (param.dbn.nChord - 1) * cgivenk(iKey, iThis);
                end
            end
        end
    end
end
bnet.CPD{nodenumber} = tabular_CPD(bnet,nodenumber,helper_table);

%-------------------------------------------------------
nodename = 'bass';
% (1) old chord
% (2) new chord
% (3) this bass
nodenumber = bnet.names(nodename) + bnet.nnodes_per_slice;
fprintf(1,'    (%d) %s (slice 2) \n', nodenumber, nodename);
helper_table = get_field(tabular_CPD(bnet,nodenumber,'CPT','unif'),'cpt');
for iOldchord = 1:param.dbn.nChord
    for iNewchord = 1:param.dbn.nChord
        if iOldchord ~= iNewchord % first beat of new chord
            for iThisbass = 1:param.dbn.nBass
                if iThisbass == param.dbn.chordbassnotes(iNewchord) % correct bass note
                    helper_table(iOldchord, iNewchord, iThisbass) = param.dbn.nominalbass;
                else
                    helper_table(iOldchord, iNewchord, iThisbass) = (1 - param.dbn.nominalbass) / (param.dbn.nBass-1);
                end
            end
        else
            for iThisbass = 1:param.dbn.nBass
                if ~param.dbn.basshaschordnotes
                    if iThisbass == param.dbn.chordbassnotes(iNewchord) % correct bass note
                        helper_table(iOldchord, iNewchord, iThisbass) = param.dbn.nominalbass_inner;
                    else
                        helper_table(iOldchord, iNewchord, iThisbass) = (1 - param.dbn.nominalbass_inner) / (param.dbn.nBass-1);
                    end
                else
                    chordnbass = [param.dbn.chordpcs(:,iNewchord); 0];
                    chordnbass(param.dbn.chordbassnotes(iNewchord)) = chordnbass(param.dbn.chordbassnotes(iNewchord))+1;
                    if ismember(iThisbass, find(chordnbass)) % chord bass note
                        helper_table(iOldchord, iNewchord, iThisbass) = param.dbn.nominalbass_inner * chordnbass(iThisbass) / sum(chordnbass);
                    else
                        helper_table(iOldchord, iNewchord, iThisbass) = (1 - param.dbn.nominalbass_inner) / (param.dbn.nBass-sum(chordnbass)-1);
                    end
                end
            end
        end
    end
end

bnet.CPD{nodenumber} = tabular_CPD(bnet,nodenumber,helper_table);



function [cgivenk, param] = chordgivenkey(param)
switch param.dbn.keyset
    case 'major'
        profile = [1 0 1 0 1 1 0 1 0 1 0 1];
    case 'majorminor'
        profile = [1 0 1 0 1 1 0 1 0 1 0 1;
                   1 0 1 1 0 1 0 1 1 1 1 1];
end

nMode = size(profile,1);

profiles = zeros(12 * nMode, 12);
param.dbn.nKey = nMode * 12;

for iMode = 1:nMode
    for iSemitone = 1:12
        profiles(12 * (iMode-1) + iSemitone,:) = ...
            circshift(profile(iMode,:),[0,iSemitone-1]);
    end
end

cgivenk = zeros([param.dbn.nKey,param.dbn.nChord]);

% for iKey = 1:param.dbn.nKey
%     for iChord = 1:param.dbn.nChord
%         chord_prefit = ...
%             (1-profiles(iKey,:)) * param.dbn.chordpcs(:,iChord);
%         cgivenk(iKey,iChord) = 1 / (chord_prefit + param.dbn.key_c);
%     end
% end


for iKey = 1:param.dbn.nKey
    for iChord = 1:param.dbn.nChord
        chord_prefit = ...
            (1-profiles(iKey,:)) * param.dbn.chordpcs(:,iChord);
        if chord_prefit == 0 && mod(iKey-iChord,12) == 0 % is in key and key and chord share root
            chord_chord_prefit = 2;
        else
            chord_chord_prefit = 1;
        end
        cgivenk(iKey,iChord) = (chord_chord_prefit / exp(chord_prefit + 0.0997)) * 0.0363 + 0.0667;
    end
end

cgivenk = qnormalise(cgivenk,1,2);


