function param = param_mirex(chromafilename)
param = param_skeleton();
param.beattype = 'auto';
param.metafilename = chromafilename;

param.dsp.extractiontype = 'vampprecalculated';
param.dsp.resampleto = 11025;
param.dsp.blocksize = 4096;
param.dsp.hopsize = 512;
param.dsp.nSbin = 3;
param.dsp.nrmsmoothlen = 1; % no norm smoothing
param.dsp.separatenormalising = true;
param.dsp.mutesilent = true;
param.dsp.norm = 'max';
param.dsp.nochord05 = true;

param.newchroma.whitemethod = 'standardize';
param.newchroma.paling = [];
param.newchroma.partials1 = 0.6;
param.newchroma.partials2 = 0.6;

param.dbn.infermpe = true;
param.dbn.name = 'ISMIR_2010'; % name of the model 
param.dbn.nMeter = 1; % number of different meters
param.dbn.nMetpos = 4; % number of metric positions (in the different meters)
param.dbn.nBass = 13; 
param.dbn.keyset = 'majorminor';
param.dbn.keynames = {'C','Db','D','Eb','E','F','F#','G','Ab','A','Bb','B',...
    'C:minor','Db:minor','D:minor','Eb:minor','E:minor','F:minor','F#:minor','G:minor','Ab:minor','A:minor','Bb:minor','B:minor'}; % set of key names 
param.dbn.lilykeynames = {'c \major','des \major','d \major','es \major','e \major','f \major','fis \major','g \major','aes \major','a \major','bes \major','b \major',...
    'c \minor','des \minor','d \minor','es \minor','e \minor','f \minor','fis \minor','g \minor','aes \minor','a \minor','bes \minor','b \minor'}; % set of key names 
param.dbn.key_c = 4.83; % strange variable c for expert function
param.dbn.nKey = 24;
param.dbn.chordclassset = {'','/3','/5',':maj6',':maj7',':min',':7',':min7',':dim',':aug','N'}; % set of chord classes
% param.dbn.nChord = 121;
param.dbn.chordpcs = []; % chord pitch class set (binary vector of length 12)
param.dbn.basshaschordnotes = true;
param.dbn.chromavar = 0.2; % variance of the chroma vector, applicable in manually set model
param.dbn.basschromavar = 0.1; % same as above for the bass
param = add_chords(param); % adding chord parameters
param.dbn.beattrans = 0.95; % transition probability to next beat (manually set)
param.dbn.keychange = 0.08; % key change probability (manually set)
param.dbn.nominalbass = 0.8; % probability of nominal bass pc sounding on first beat of chord
param.dbn.nominalbass_inner = 0.8; % as above, but on the rest of the beats of a chord
param.dbn.chordchange = 1 - [.5 .1 .4 .1]; % one minus prob of chord change at different beats
param.dbn.inferencebypart = true;

param.seg.type = 'auto'; % segmentation with editor intervention, or no segmentation, etc.
param.seg.quantile = .1;
param.seg.editor = false;
param.seg.beat_pairs = false;
param.seg.medfilt_length = 5;
param.seg.barline_spacing = 1;
param.seg.thresh_beat = .85;
param.seg.minlength = 28;
param.seg.maxlength = 128;
param.seg.standardise = false;
param.seg.thresh_seg = .8;
param.seg.precalc = 0;
param.seg.newcheck = false;
% param.seg.inferencebypart = true;
if param.seg.newcheck
    param.seg.segext = '.seg';
else
    param.seg.segext = '.segold';
end
