function song = song_syncchroma(song, param)

[song.syncchroma.treble, treblenrm] = syncchroma(song.chroma.treble, song.beattimes, song.t);
[song.syncchroma.bass, bassnrm] = syncchroma(song.chroma.bass, song.beattimes, song.t(:));

song.syncchroma.bass(isnan(sum(song.syncchroma.bass,2)),:) = 1;
song.syncchroma.treble(isnan(sum(song.syncchroma.treble,2)),:) = 1;

song.syncchroma.bass(sum(song.syncchroma.bass,2)<.01,:) = 1;
song.syncchroma.treble(sum(song.syncchroma.treble,2)<.01,:) = 1;

bcnrm = qnorm(song.syncchroma.bass,1,2);
if param.dsp.bassbin13 % if the model requires a thirteenth bass bin
    bc = qnormalise(song.syncchroma.bass,1,2);
    bc13 = (12 * max(bc,[],2)).^(-2) .* bcnrm;
    bc13(isnan(bc13)) = 1;
    song.syncchroma.bass = [song.syncchroma.bass, bc13];
end

switch param.dsp.norm
    case 'max'
        song.syncchroma.bass = qnormalise(song.syncchroma.bass,inf,2);
        song.syncchroma.treble = qnormalise(song.syncchroma.treble,inf,2);
    case 'L1'
        song.syncchroma.bass = qnormalise(song.syncchroma.bass,1,2);
        song.syncchroma.treble = qnormalise(song.syncchroma.treble,1,2);
    case 'L2'
        disp('L2 norm')
        song.syncchroma.bass = qnormalise(song.syncchroma.bass,2,2);
        song.syncchroma.treble = qnormalise(song.syncchroma.treble,2,2);
    case 'standardize'
        song.syncchroma.bass = standardize(song.syncchroma.bass);
        song.syncchroma.treble = standardize(song.syncchroma.treble);
    case 'meta'
        song.syncchroma.bass = qnormalise(song.syncchroma.bass,inf,2);
        disp('no standardization for treble chroma; max normalisation for bass')
    otherwise
        'blub'
end

% make all NaNs to 1
song.syncchroma.bass(isnan(sum(song.syncchroma.bass,2)),:) = 1;
song.syncchroma.treble(isnan(sum(song.syncchroma.treble,2)),:) = 1;
% aah, what does this do???
% song.syncchroma.bass(sum(song.syncchroma.bass,2) < 1,:) = 1;
% song.syncchroma.treble(sum(song.syncchroma.treble,2) < 1,:) = 1;




% below "wide" chroma processing, that's neglected ...
% try
%     [song.syncchroma.wide, widenrm] = syncchroma(song.chroma.wide, song.beattimes, song.t);
%     for iBeat = 1:song.nBeat
%         song.syncchroma.wide(iBeat,:) = song.syncchroma.wide(iBeat,:) / widenrm(iBeat);
%     end
% catch
%     warning('could not find wide chroma');
% end



function [bc, nrm] = syncchroma(chromag,beats,frame_t)
% assigning beats to frames
nBeat = size(beats,1);
nFrame = length(frame_t);

bc = ones(nBeat,size(chromag,1));
chromag(:,isnan(sum(chromag))) = 1;

if size(beats,2) < 2
    for iBeat = 1:nBeat-1
        currind = find(frame_t >= beats(iBeat) & frame_t < beats(iBeat+1));
        bc(iBeat,:) = median(chromag(:,currind),2);
%         if ~any(isnan(bc(iBeat,:)))
%             bc(iBeat,:) = bc(iBeat,:)/max(bc(iBeat,:));
%         end
    end
else
    for iBeat = 1:nBeat
        currind = find(frame_t >= beats(iBeat,1) & frame_t < beats(iBeat,2));
        bc(iBeat,:) = median(chromag(:,currind),2)';
    end
end
nrm = qnorm(bc,inf,2);
% bc = qnormalise(bc,inf,2);
